#!python
'''
    Title: img2vdu
    Author: TheOuterLinux
    Purpose: To convert modern image formats -- such as PNG, JPEG, GIF, 
             etc., into a scriptable format for BBC Micro computers, or 
             BBC BASIC in general.
    
    Last updated: 2021/03/06

    License:
    --------
    Copyright (C) 2020-2021 TheOuterLinux

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.

    img2vdu.py requirements:
    ----------------------
        ImageMagick
        Python2.7 or Python3
            Modules:
                PIL (Python 2.7 - 3.5 via pip) or python3-pillowfight (Python 3.6 +)
                numpy
                ptyprocess
                six
                colorama
                
    This script has only been tested on GNU/Linux, but was written in
    a way that should hopefully work just fine on MacOS or Windows.
'''
import os
from os import system
import sys
import getopt
import signal
import operator
from PIL import Image
import subprocess
import numpy as np
from subprocess import call
import tempfile
from tempfile import gettempdir
from shutil import rmtree
from six.moves import input as raw_input
from colorama import init, Fore, Back, Style

import signal
import time
import sys

def clear(): 
    # check and make call for specific operating system 
    _ = call('clear' if os.name =='posix' else 'cls') 

#Filter ANSI escape sequences out of any text sent to stdout or stderr, 
#and replace them with equivalent Win32 calls if using Windows.
if os.name == 'Windows': init()

#Create temporary directory to work in.
#This method makes things much more cross-platform.
tmp = os.path.join(gettempdir(), 'img2vdu_{}'.format(hash(os.times())))
os.makedirs(tmp)

#Create variable DIR if needing to make path relative to Python script location
DIR = os.path.dirname(os.path.realpath(__file__))

#Clear the terminal screen
clear()

#Title
if os.name == 'Windows':
	myCoolTitle='img2vdu.py by TheOuterLinux'
	system('title '+myCoolTitle)
else:
	 sys.stdout.write('\x1b]2;img2vdu.py by TheOuterLinux\x07')
print (' ' + Fore.YELLOW + Back.YELLOW + '+-------------------------------------------------------------------++' + Style.RESET_ALL)
print (' ' + Fore.YELLOW + Back.YELLOW + '||'+ Fore.BLACK + Back.WHITE + ' img2vdu.py - Convert a modern image to BBC BASIC VDU 23 format   ' + Fore.YELLOW + Back.YELLOW + '||' + Style.RESET_ALL)
print (' ' + Fore.YELLOW + Back.YELLOW + '||'+ Fore.BLACK + Back.WHITE + ' Author - TheOuterLinux (https://theouterlinux.gitlab.io)         ' + Fore.YELLOW + Back.YELLOW + '||' + Style.RESET_ALL)
print (' ' + Fore.YELLOW + Back.YELLOW + '||'+ Fore.BLACK + Back.WHITE + ' License - GPL v3 or newer (https://www.gnu.org/licenses/)        ' + Fore.YELLOW + Back.YELLOW + '||' + Style.RESET_ALL)
#print (' ' + Fore.YELLOW + Back.YELLOW + '+-------------------------------------------------------------------++' + Style.RESET_ALL)
print (' ' + Fore.YELLOW + Back.YELLOW + "||"+ Fore.WHITE + Back.YELLOW + "           Use 'q' or 'quit' any time to end program.             " + Fore.YELLOW + Back.YELLOW + "||" + Style.RESET_ALL)
print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK)

#Print temporarily created palette locations
print(' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' Temporary folder created in ' + Fore.WHITE + tmp + Fore.YELLOW + '.')
print(' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '')
     
# Ask for the full path of the file to convert. The xfce4-terminal
# browser supports drag-and-drop.
filepath = raw_input(' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' File to convert (full path): ' + Style.RESET_ALL)
filepath = filepath.replace("'", '')
filepath = filepath.strip()
    
if filepath in ['q','quit','Q','Quit','QUIT','Exit','EXIT','exit']:
	rmtree(tmp, ignore_errors=True)
	print('')
	exit()
if os.path.exists(filepath):
	print(' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' ' + Style.RESET_ALL)
else:
	clear()
	print (' ' + Fore.YELLOW + Back.YELLOW + '+-------------------------------------------------------------------++' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + '||'+ Fore.BLACK + Back.WHITE + ' img2vdu.py - Convert a modern image to BBC BASIC VDU 23 format   ' + Fore.YELLOW + Back.YELLOW + '||' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + '||'+ Fore.BLACK + Back.WHITE + ' Author - TheOuterLinux (https://theouterlinux.gitlab.io)         ' + Fore.YELLOW + Back.YELLOW + '||' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + '||'+ Fore.BLACK + Back.WHITE + ' License - GPL v3 or newer (https://www.gnu.org/licenses/)        ' + Fore.YELLOW + Back.YELLOW + '||' + Style.RESET_ALL)
	#print (' ' + Fore.YELLOW + Back.YELLOW + '+-------------------------------------------------------------------++' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + "||"+ Fore.WHITE + Back.YELLOW + "           Use 'q' or 'quit' any time to end program.             " + Fore.YELLOW + Back.YELLOW + "||" + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '                                                                    ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' You must input an existing full file path to use this program.     ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' Please use one of the following formats:                           ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' ----------------------------------------                           ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '     *.blp                 *.pcx                                    ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '     *.bmp                 *.pdf                                    ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '     *.cur                 *.png                                    ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '     *.gbr                 *.ppm                                    ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '     *.gif                 *.psd                                    ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '     *.icns                *.tga                                    ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '     *.ico                 *.tif, tiff                              ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '     *.jp2 (JPEG 2000)     *.wal                                    ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '     *.jpg, jpeg           *.webp                                   ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '     *.mic                 *.xbm                                    ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '     *.mps                 *.xpm                                    ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '     *.palm                *.xv                                     ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '                                                                    ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + '                                                                      ' + Back.BLACK + ' ')
	print ('' + Style.RESET_ALL)
	rmtree(tmp, ignore_errors=True)
	exit()

# Convert file extensions to all lower-case since older FAT file systems
# tend to use all upper-case while newer file systems can use upper and
# lower-case. Avoids the hassle of writing more if-than statements than
# necessary.
fileext = os.path.splitext(filepath.lower())[1]
extcount = len(fileext)

# Get the file name without the full path or extension (in most cases)
base = os.path.basename(filepath)
base = os.path.splitext(base)[0]

# Get the path of the directory the original file is in so we know where 
# to put the converted result.
dirpath = os.path.dirname(filepath)

# The file name and path to use for a preview file
previewfile = dirpath + '/' + base + '_img2vdu_preview.gif'

# Prepare the file name to make it DOS-friendly by removing
# spaces and shortening the file name to 8 characters. Using 'qbd' as
# the file extension to stand for 'QuickBASIC DATA'. And because it is
# still plain text, it shouldn't trick the computer into thinking it is
# a Quicken file.
dosbase = base.replace(' ', '')
dosbase = dosbase[:8]
dosbase = dosbase.upper()
#vdufilepath = tmp + '/' + dosbase + '.VDU'
#vdufilepathfinal = dirpath + '/' + dosbase + '.VDU'

# The PIL library supports more formats than this, but only supporting
# the following makes the most sense.
if  fileext in ['.blp', '.bmp', '.cur', '.gbr', '.gif', '.icns', '.ico', \
                '.jp2', '.jpeg', '.jpg', '.mic', '.msp', '.palm', '.pcx', \
                '.pdf', '.png', '.ppm', '.psd', '.tga', '.tif', '.tiff', \
                '.wal', '.webp', '.xbm', '.xpm', '.xv']:
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' File format ' + Fore.WHITE + fileext + Fore.YELLOW + ' supported.')
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '')
else:
	clear()
	print (' ' + Fore.YELLOW + Back.YELLOW + '+-------------------------------------------------------------------++' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + '||'+ Fore.BLACK + Back.WHITE + ' img2vdu.py - Convert a modern image to BBC BASIC VDU 23 format   ' + Fore.YELLOW + Back.YELLOW + '||' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + '||'+ Fore.BLACK + Back.WHITE + ' Author - TheOuterLinux (https://theouterlinux.gitlab.io)         ' + Fore.YELLOW + Back.YELLOW + '||' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + '||'+ Fore.BLACK + Back.WHITE + ' License - GPL v3 or newer (https://www.gnu.org/licenses/)        ' + Fore.YELLOW + Back.YELLOW + '||' + Style.RESET_ALL)
	#print (' ' + Fore.YELLOW + Back.YELLOW + '+-------------------------------------------------------------------++' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + "||"+ Fore.WHITE + Back.YELLOW + "           Use 'q' or 'quit' any time to end program.             " + Fore.YELLOW + Back.YELLOW + "||" + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '                                                                    ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	if extcount == 0: print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' You must input an existing full file path to use this program.     ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	if extcount == 1: print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' File format *' + fileext + ' not supported.                                      ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	if extcount == 2: print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' File format *' + fileext + ' not supported.                                     ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	if extcount == 3: print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' File format *' + fileext + ' not supported.                                    ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	if extcount == 4: print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' File format *' + fileext + ' not supported.                                   ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	if extcount == 5: print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' File format *' + fileext + ' not supported.                                  ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' Please use one of the following formats:                           ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' ----------------------------------------                           ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '     *.blp                 *.pcx                                    ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '     *.bmp                 *.pdf                                    ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '     *.cur                 *.png                                    ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '     *.gbr                 *.ppm                                    ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '     *.gif                 *.psd                                    ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '     *.icns                *.tga                                    ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '     *.ico                 *.tif, tiff                              ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '     *.jp2 (JPEG 2000)     *.wal                                    ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '     *.jpg, jpeg           *.webp                                   ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '     *.mic                 *.xbm                                    ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '     *.mps                 *.xpm                                    ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '     *.palm                *.xv                                     ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '                                                                    ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + '                                                                      ' + Back.BLACK + ' ')
	print ('' + Style.RESET_ALL)
	rmtree(tmp, ignore_errors=True)
	exit()

# Get the selected image file's width and height
im = Image.open(filepath)
width, height = im.size
colorcount = len(list(filter(None, im.histogram())))
if sys.version_info[0] == 2:
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' Image width:' + Fore.WHITE), width
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' Image height:' + Fore.WHITE), height
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' Image color count:' + Fore.WHITE), colorcount
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' ')
	if width > 72 and height < 256:
		print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + Fore.RED + ' Your image width is larger than 72 pixels.' + Fore.WHITE)
		print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + Fore.RED + ' It is currently recommended that you use this script for smaller graphics.' + Fore.WHITE)
		print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + Fore.RED + ' You can use the imgsplit.py script to split the graphic into 72x72 pieces.' + Fore.WHITE)
	if height > 256 and width < 72:
		print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + Fore.RED + ' Your image height is larger than 256 pixels.' + Fore.WHITE)
		print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + Fore.RED + ' It is currently recommended that you use this script for smaller graphics.' + Fore.WHITE)
		print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + Fore.RED + ' You can use the imgsplit.py script to split the graphic into 72x72 pieces.' + Fore.WHITE)
	if width > 72 and height > 256:
		print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + Fore.RED + ' Your image width is larger than 72 pixels and your image height is larger than 256 pixels.' + Fore.WHITE)
		print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + Fore.RED + ' It is currently recommended that you use this script for smaller graphics.' + Fore.WHITE)
		print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + Fore.RED + ' You can use the ' + Fore.WHITE + 'imgsplit.py' + Fore.RED + ' script to split the graphic into 72x72 pieces.' + Fore.WHITE)

# Ask what MODE will be used
# Modes commented-out are not yet supported but may be one day and I'm
# too lazy to look-up their specs again.
print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '')
print (' ' + Fore.WHITE + Back.YELLOW + ' ' + Back.BLACK + ' MODES:')
print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' ------')
print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' MODE ' + Fore.WHITE + '0' + Fore.YELLOW + ' - 80x32 characters;  2 colors; 20K memory')
# print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' MODE ' + Fore.WHITE + '1' + Fore.YELLOW + ' - 40x32 characters;  4 colors; 20K memory')
# print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' MODE ' + Fore.WHITE + '2' + Fore.YELLOW + ' - 20x32 characters;  8 normal and 8 flashing colors; 20K memory')
print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' MODE ' + Fore.WHITE + '3' + Fore.YELLOW + ' - 80x25 characters;  2 color text-only; 16K memory')
print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' MODE ' + Fore.WHITE + '4' + Fore.YELLOW + ' - 40x32 characters;  2 colors; 10K memory')
# print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' MODE ' + Fore.WHITE + '5' + Fore.YELLOW + ' - 20x32 characters;  4 colors; 10K memory')
print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' MODE ' + Fore.WHITE + '6' + Fore.YELLOW + ' - 40x25 characters;  2 color text only; 8K memory')
print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' MODE ' + Fore.WHITE + '7' + Fore.YELLOW + ' - 40x25 characters;  teletext (ASCII art); characters are not editable; 1K memory')
print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '')
screenmode = raw_input(' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' What ' + Fore.WHITE + 'MODE' + Fore.YELLOW + ' do you plan on using? (' + Fore.WHITE + '0,1,..,q' + Fore.YELLOW + '): ' + Style.RESET_ALL)
screenmode = screenmode.lower()
clear()

print(' ' + Fore.YELLOW + Back.YELLOW + '+-----------------------------------------------------------------++' + Style.RESET_ALL)
print (' ' + Fore.YELLOW + Back.YELLOW + '||'+ Fore.BLACK + Back.WHITE + ' img2vdu.py - Convert a modern image to BBC BASIC VDU 23 format ' + Fore.YELLOW + Back.YELLOW + '||' + Style.RESET_ALL)
print(' ' + Fore.YELLOW + Back.YELLOW + '||'+ Fore.BLACK + Back.WHITE + ' Author - TheOuterLinux (https://theouterlinux.gitlab.io)       ' + Fore.YELLOW + Back.YELLOW + '||' + Style.RESET_ALL)
print(' ' + Fore.YELLOW + Back.YELLOW + '||'+ Fore.BLACK + Back.WHITE + ' License - GPL v3 or newer (https://www.gnu.org/licenses/)      ' + Fore.YELLOW + Back.YELLOW + '||' + Style.RESET_ALL)
#print (' ' + Fore.YELLOW + Back.YELLOW + '+-------------------------------------------------------------------++' + Style.RESET_ALL)
print(' ' + Fore.YELLOW + Back.YELLOW + "||"+ Fore.WHITE + Back.YELLOW + "           Use 'q' or 'quit' any time to end program.           " + Fore.YELLOW + Back.YELLOW + "||" + Style.RESET_ALL)
print('')
print(Style.RESET_ALL)
def screen():
	    print (' ' + Back.YELLOW + ' ' + Back.BLACK + Fore.WHITE + ' It seems as though your image is quite large (' + Fore.RED + '> ' + str(screenw) + 'x' + str(screenh) + Fore.WHITE + ').         ' + Back.YELLOW + ' ' + Back.RED + ' ' + Style.RESET_ALL)
	    print (' ' + Back.YELLOW + ' ' + Back.BLACK + Fore.WHITE + ' If you plan on creating with ' + Fore.YELLOW + 'Brandy' + Fore.WHITE + ' and then later opening      ' + Back.YELLOW + ' ' + Back.RED + ' ' + Style.RESET_ALL)
	    print (' ' + Back.YELLOW + ' ' + Back.BLACK + Fore.WHITE + " with " + Fore.YELLOW +"a BBC Micro" + Fore.WHITE + " or " + Fore.YELLOW + "an emulator" + Fore.WHITE + ", things will not look right.  " + Fore.RED + "  " + Back.YELLOW + ' ' + Back.RED + ' ' + Style.RESET_ALL)
	    print (' ' + Back.YELLOW + ' ' + Back.BLACK + Fore.WHITE + ' So, it is ' + Fore.RED + 'suggested that      ' + Back.YELLOW + ' ' + Back.RED + ' ' + Style.RESET_ALL)
	    print (' ' + Back.YELLOW + ' ' + Back.BLACK + Fore.WHITE + ' ' + Fore.RED + 'you shrink the file' + Fore.WHITE + ' and run this script again. However,         ' + Back.YELLOW + ' ' + Back.RED + ' ' + Style.RESET_ALL)
	    print (' ' + Back.YELLOW + ' ' + Back.BLACK + Fore.WHITE + ' who am I to stop you. Stretching in the following case refers   ' + Back.YELLOW + ' ' + Back.RED + ' ' + Style.RESET_ALL)
	    print (' ' + Back.YELLOW + ' ' + Back.BLACK + Fore.WHITE + ' to ' + Fore.RED + 'forcing' + Fore.WHITE + ' a width and height of ' + Fore.RED + screendim + Fore.WHITE + ' as opposed to keeping  ' + Back.YELLOW + ' ' + Back.RED + ' ' + Style.RESET_ALL)
	    print (' ' + Back.YELLOW + ' ' + Back.BLACK + Fore.WHITE + ' the original aspect ratio of the image.                         ' + Back.YELLOW + ' ' + Back.RED + ' ' + Style.RESET_ALL)
	    print (' ' + Back.YELLOW + '                                                                   ' + Back.RED + ' ' + Style.RESET_ALL)
	    print (' ' + Back.BLACK + ' ' + Back.RED + '                                                                   ' + Style.RESET_ALL)
	    print('')

#MODE 0
if screenmode in ['0','mode 0']:
	print (' ' + Back.YELLOW + '                       ' + Style.RESET_ALL)
	print (' ' + Fore.RED + Back.YELLOW +' ' + Back.WHITE + ' MODE 0 selected...  ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Back.YELLOW + '                                                                   ' + Style.RESET_ALL)
	colornumber = 2
	screenw = 640
	charw = 80
	screenh = 256
	charh = 32
	screendim = str(screenw) + 'x' + str(screenh)
	if width > screenw or height > screenh:
		screen()
		stretchyesno = raw_input(' ' + Fore.YELLOW + ' Would you like to force a ' + screendim + ' pixel image? (' + Fore.WHITE + 'y/N,q' + Fore.YELLOW + '): ' + Style.RESET_ALL)
		stretchyesno = stretchyesno.lower()
		if stretchyesno in ('y', 'yes'): dimensions = '-resize ' + screendim + '\!'
		elif stretchyesno in ('q','quit','exit'):
			rmtree(tmp, ignore_errors=True)
			print('')
			exit()
		else: dimensions = '-resize ' + screendim
	else:
		stretchyesno = 'no'
		#Ensures that the width is divisible by 8
		width = int( 8 * round( width / 8. ))
		dimensions = '-resize '+str(width)+'x'+str(height)+'\!'

#MODE 3
elif screenmode in ['3','mode 3']:
	screenmode = '3'
	print (' ' + Back.YELLOW + '                       ' + Style.RESET_ALL)
	print (' ' + Fore.RED + Back.YELLOW +' ' + Back.WHITE + ' MODE 3 selected...  ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Back.YELLOW + '                                                                   ' + Style.RESET_ALL)
	colornumber = 2
	screenw = 640
	charw = 80
	screenh = 200
	charh = 25
	screendim = str(screenw) + 'x' + str(screenh)
	if width > screenw or height > screenh:
		screen()
		stretchyesno = raw_input(' ' + Fore.YELLOW + ' Would you like to force a ' + screendim + ' pixel image? (' + Fore.WHITE + 'y/N,q' + Fore.YELLOW + '): ' + Style.RESET_ALL)
		stretchyesno = stretchyesno.lower()
		if stretchyesno in ('y', 'yes'): dimensions = '-resize ' + screendim + '\!'
		elif stretchyesno in ('q','quit','exit'):
			rmtree(tmp, ignore_errors=True)
			print('')
			exit()
		else: dimensions = '-resize ' + screendim
	else:
		stretchyesno = 'no'
		#Ensures that the width is divisible by 8
		width = int( 8 * round( width / 8. ))
		dimensions = '-resize '+str(width)+'x'+str(height)+'\!'

#MODE 4
elif screenmode in ['4','mode 4']:
	screenmode = '4'
	print (' ' + Back.YELLOW + '                       ' + Style.RESET_ALL)
	print (' ' + Fore.RED + Back.YELLOW +' ' + Back.WHITE + ' MODE 4 selected...  ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Back.YELLOW + '                                                                   ' + Style.RESET_ALL)
	colornumber = 2
	screenw = 320
	charw = 40
	screenh = 256
	charh = 32
	screendim = str(screenw) + 'x' + str(screenh)
	if width > screenw or height > screenh:
		screen()
		stretchyesno = raw_input(' ' + Fore.YELLOW + ' Would you like to force a ' + screendim + ' pixel image? (' + Fore.WHITE + 'y/N,q' + Fore.YELLOW + '): ' + Style.RESET_ALL)
		stretchyesno = stretchyesno.lower()
		if stretchyesno in ('y', 'yes'): dimensions = '-resize ' + screendim + '\!'
		elif stretchyesno in ('q','quit','exit'):
			rmtree(tmp, ignore_errors=True)
			print('')
			exit()
		else: dimensions = '-resize ' + screendim
	else:
		stretchyesno = 'no'
		#Ensures that the width is divisible by 8
		width = int( 8 * round( width / 8. ))
		dimensions = '-resize '+str(width)+'x'+str(height)+'\!'

#MODE 6
elif screenmode in ['6','mode 6']:
	screenmode = '6'
	print (' ' + Back.YELLOW + '                       ' + Style.RESET_ALL)
	print (' ' + Fore.RED + Back.YELLOW +' ' + Back.WHITE + ' MODE 6 selected...  ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Back.YELLOW + '                                                                   ' + Style.RESET_ALL)
	colornumber = 2 #It's actually 4, but I haven't figured that out yet
	screenw = 320
	charw = 40
	screenh = 200
	charh = 25
	screendim = str(screenw) + 'x' + str(screenh)
	if width > screenw or height > screenh:
		screen()
		stretchyesno = raw_input(' ' + Fore.YELLOW + ' Would you like to force a ' + screendim + ' pixel image? (' + Fore.WHITE + 'y/N,q' + Fore.YELLOW + '): ' + Style.RESET_ALL)
		stretchyesno = stretchyesno.lower()
		if stretchyesno in ('y', 'yes'): dimensions = '-resize ' + screendim + '\!'
		elif stretchyesno in ('q','quit','exit'):
			rmtree(tmp, ignore_errors=True)
			print('')
			exit()
		else: dimensions = '-resize ' + screendim
	else:
		stretchyesno = 'no'
		#Ensures that the width is divisible by 8
		width = int( 8 * round( width / 8. ))
		dimensions = '-resize '+str(width)+'x'+str(height)+'\!'

#MODE 7
elif screenmode in ['7','mode 7']:
	screenmode = '7'
	print (' ' + Back.YELLOW + '                       ' + Style.RESET_ALL)
	print (' ' + Fore.RED + Back.YELLOW +' ' + Back.WHITE + ' MODE 7 selected...  ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Back.YELLOW + '                                                                   ' + Style.RESET_ALL)
	colornumber = 0 #Teletext
	screenw = 40
	charw = 320
	screenh = 200
	charh = 25
	screendim = str(screenw) + 'x' + str(screenh)
	if width > screenw or height > screenh:
		screen()
		stretchyesno = raw_input(' ' + Fore.YELLOW + ' Would you like to force a ' + screendim + ' pixel image? (' + Fore.WHITE + 'y/N,q' + Fore.YELLOW + '): ' + Style.RESET_ALL)
		stretchyesno = stretchyesno.lower()
		if stretchyesno in ('y', 'yes'): dimensions = '-resize ' + screendim + '\!'
		elif stretchyesno in ('q','quit','exit'):
			rmtree(tmp, ignore_errors=True)
			print('')
			exit()
		else: dimensions = '-resize ' + screendim
	else:
		stretchyesno = 'no'
		#Ensures that the width is divisible by 8
		width = int( 8 * round( width / 8. ))
		dimensions = '-resize '+str(width)+'x'+str(height)+'\!'

#QUIT
elif screenmode in ['q','quit','Q','Quit','QUIT','Exit','EXIT','exit']: exit()

#If invalid MODE picked
else:
	print (' ' + Back.YELLOW + '                       ' + Style.RESET_ALL)
	print (' ' + Fore.RED + Back.YELLOW +' ' + Back.WHITE + ' MODE 0 selected...  ' + Back.YELLOW + ' ' + Style.RESET_ALL)
	print (' ' + Back.YELLOW + '                                                                   ' + Style.RESET_ALL)
	colornumber = 2
	screenw = 640
	charw = 80
	screenh = 256
	charh = 32
	screendim = str(screenw) + 'x' + str(screenh)
	if width > screenw or height > screenh:
		screen()
		stretchyesno = raw_input(' ' + Fore.YELLOW + ' Would you like to force a ' + screendim + ' pixel image? (' + Fore.WHITE + 'y/N,q' + Fore.YELLOW + '): ' + Style.RESET_ALL)
		stretchyesno = stretchyesno.lower()
		if stretchyesno in ('y', 'yes'): dimensions = '-resize ' + screendim + '\!'
		elif stretchyesno in ('q','quit','exit'):
			rmtree(tmp, ignore_errors=True)
			print('')
			exit()
		else: dimensions = '-resize ' + screendim
	else:
		stretchyesno = 'no'
		#Ensures that the width is divisible by 8
		width = int( 8 * round( width / 8. ))
		dimensions = '-resize '+str(width)+'x'+str(height)+'\!'

# Ask the user about what column and row to start printing the graphic.
# Columns are like 'X' position but for the cursor, while the row would
# be like 'Y'. Keep in mind that the start of the column and row positions
# is based off of the top-left corner of the graphic and the width and 
# height of the graphic must be taken into account when positioning, such
# as trying to place it in the center of the screen.
clear()
print(' ' + Fore.YELLOW + Back.YELLOW + '+-----------------------------------------------------------------++' + Style.RESET_ALL)
print (' ' + Fore.YELLOW + Back.YELLOW + '||'+ Fore.BLACK + Back.WHITE + ' img2vdu.py - Convert a modern image to BBC BASIC VDU 23 format ' + Fore.YELLOW + Back.YELLOW + '||' + Style.RESET_ALL)
print(' ' + Fore.YELLOW + Back.YELLOW + '||'+ Fore.BLACK + Back.WHITE + ' Author - TheOuterLinux (https://theouterlinux.gitlab.io)       ' + Fore.YELLOW + Back.YELLOW + '||' + Style.RESET_ALL)
print(' ' + Fore.YELLOW + Back.YELLOW + '||'+ Fore.BLACK + Back.WHITE + ' License - GPL v3 or newer (https://www.gnu.org/licenses/)      ' + Fore.YELLOW + Back.YELLOW + '||' + Style.RESET_ALL)
#print (' ' + Fore.YELLOW + Back.YELLOW + '+-------------------------------------------------------------------++' + Style.RESET_ALL)
print(' ' + Fore.YELLOW + Back.YELLOW + "||"+ Fore.WHITE + Back.YELLOW + "           Use 'q' or 'quit' any time to end program.           " + Fore.YELLOW + Back.YELLOW + "||" + Style.RESET_ALL)
print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '')
cursorw = raw_input(' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' Start printing graphic on what column (X)? (' + Fore.WHITE + '1, 2, ...'+str(charw)+' is max; 1 is default' + Fore.YELLOW + '): ' + Style.RESET_ALL)
if cursorw == '': cursorw = 1

clear()
print(' ' + Fore.YELLOW + Back.YELLOW + '+-----------------------------------------------------------------++' + Style.RESET_ALL)
print (' ' + Fore.YELLOW + Back.YELLOW + '||'+ Fore.BLACK + Back.WHITE + ' img2vdu.py - Convert a modern image to BBC BASIC VDU 23 format ' + Fore.YELLOW + Back.YELLOW + '||' + Style.RESET_ALL)
print(' ' + Fore.YELLOW + Back.YELLOW + '||'+ Fore.BLACK + Back.WHITE + ' Author - TheOuterLinux (https://theouterlinux.gitlab.io)       ' + Fore.YELLOW + Back.YELLOW + '||' + Style.RESET_ALL)
print(' ' + Fore.YELLOW + Back.YELLOW + '||'+ Fore.BLACK + Back.WHITE + ' License - GPL v3 or newer (https://www.gnu.org/licenses/)      ' + Fore.YELLOW + Back.YELLOW + '||' + Style.RESET_ALL)
#print (' ' + Fore.YELLOW + Back.YELLOW + '+-------------------------------------------------------------------++' + Style.RESET_ALL)
print(' ' + Fore.YELLOW + Back.YELLOW + "||"+ Fore.WHITE + Back.YELLOW + "           Use 'q' or 'quit' any time to end program.           " + Fore.YELLOW + Back.YELLOW + "||" + Style.RESET_ALL)
print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '')
cursorh = raw_input(' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' Start printing graphic on what row (Y)? (' + Fore.WHITE + '1, 2, ...'+str(charh)+' is max; 1 is default' + Fore.YELLOW + '): ' + Style.RESET_ALL)
if cursorh == '': cursorh = 1

# Convert the original image to a limited color image for previewing
# before converting the preview image to VDU 23 format. GIF87 format
# was chosen for the output because it is supported by older and newer
# software where as GIF89 is mostly limited to newer software and a few
# currently maintained DOS image viewers. If the original image contains
# more colors than the target MODE supports, dithering is used.

negate=""
def previewmaker():
	if colornumber == 0:
		if colorcount > colornumber:
			cmd = 'convert ' + dimensions + ' -colors 2 -monochrome '+negate+' +antialias ' + '"'+filepath+'"' + " GIF87:" + '"'+previewfile+'"'
			subprocess.call(cmd, shell=True)
		else:
			cmd = 'convert ' + dimensions + ' -colors 2 -monochrome '+negate+' +dither +antialias ' + '"'+filepath+'"' + " GIF87:" + '"'+previewfile+'"'
			subprocess.call(cmd, shell=True)
	elif colornumber == 2:
		if colorcount > colornumber:
			cmd = 'convert ' + dimensions + ' -colors 2 -monochrome '+negate+' +antialias ' + '"'+filepath+'"' + " GIF87:" + '"'+previewfile+'"'
			subprocess.call(cmd, shell=True)
		else:
			cmd = 'convert ' + dimensions + ' -colors 2 -monochrome '+negate+' +dither +antialias ' + '"'+filepath+'"' + " GIF87:" + '"'+previewfile+'"'
			subprocess.call(cmd, shell=True)
	else:
		if colorcount > colornumber:
			cmd = 'convert ' + dimensions + ' -colors 2 -monochrome '+negate+' +antialias ' + '"'+filepath+'"' + " GIF87:" + '"'+previewfile+'"'
			subprocess.call(cmd, shell=True)
		else:
			cmd = 'convert ' + dimensions + ' -colors 2 -monochrome '+negate+' +dither +antialias ' + '"'+filepath+'"' + " GIF87:" + '"'+previewfile+'"'
			subprocess.call(cmd, shell=True)

# Run the function defined above and then check the black pixel color
# count. If it isn't more than the white colored pixels, re-run previewmaker
# but with the -negate flag to inverse the image. This should hopefully
# help save memory as well as your eyes. A black 8x8 square is essentially
# the same as using "space" (as in space-bar), which is already mapped as
# CHR$(32).
previewmaker()
colorcount_preview = len(list(filter(None, im.histogram())))
if colornumber == 2:
	black, white = Image.open(previewfile).getcolors()
	if white[0] > black[0] and width > 72:
		bwcheck = raw_input(Fore.YELLOW+'More white pixels than black was created. Would you like to invert them ('+Fore.WHITE+'y/N/q'+Fore.YELLOW+')?: '+Style.RESET_ALL)
		bwcheck = bwcheck.lower()
		if bwcheck in ['y','yes']:
			negate = '-negate'
			print ('Inverting colors before creating VDU 23...')
		elif bwcheck in ['q','quit','exit']:
			exit()
		else:
			print ('Keeping colors as they are. Moving on...')
		time.sleep(1) #Python is fast; used to give time to update thumbnail
		previewmaker()

cmd = 'convert ' + '"'+previewfile+'"' + ' -crop 8x8 -background black -extent 8x8 +repage -scene 1 GIF87:'+tmp+'/split%03d.gif'
subprocess.call(cmd, shell=True)

# Do a FOR-LOOP to create an 8x8 ones and zeros array for each 8x8 
# image within the temporary directory.
def makearray():
	im = Image.open(tmp + '/' + filename)
	rgb_im = im.convert('RGB')
	width, height = im.size
	pixelx = 1
	pixely = 1
	with open(tmp + '/' + dosbase + '_' + str(filenumber) + '.txt', "w+") as outfile:
		for pixely in range(height):
			row = []
			for pixelx in range(width):
				r,g,b = rgb_im.getpixel((pixelx, pixely))
				if [r, g, b] == [0,0,0]: row.append("0") #Black
				elif [r, g, b] == [255,255,255]: row.append("1") #White
				else: row.append("0")
			outfile.write(",".join(row) + '\n')

# Convert each text file created above in the temp directory to a VDU 23-like
# array and save each in a numpy format (.npy). 
def convertarray():
	a = np.loadtxt(tmp + '/' + dosbase + '_' + str(filenumber) + '.txt',delimiter=',',dtype='int')
	b = np.array([128,64,32,16,8,4,2,1])
	c = np.multiply(a, b)
	d = np.array([1,1,1,1,1,1,1,1])
	e = c.dot(d)
	np.save(tmp + '/' + dosbase + '_' + str(filenumber) + '.npy', e)

if colornumber == 2:
	filenumber = 224 #So that starting file number would be 241 since CHR$(240) is reserved.
	whiteblocknumber = filenumber
	blackblocknumber = 32 #32 will also work, but that is an extra character
	filelist = sorted(os.listdir(tmp)) #Process files in alphabetical order
	for filename in filelist:
		filenumber = filenumber + 1
		makearray()
	filenumber = 224
	filelist = sorted(os.listdir(tmp))
	for filename in filelist:
		if filename.endswith((".txt")):
			filenumber = filenumber + 1
			convertarray()
	clear()
	print(' ' + Fore.YELLOW + Back.YELLOW + '+-----------------------------------------------------------------++' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + '||'+ Fore.BLACK + Back.WHITE + ' img2vdu.py - Convert a modern image to BBC BASIC VDU 23 format ' + Fore.YELLOW + Back.YELLOW + '||' + Style.RESET_ALL)
	print(' ' + Fore.YELLOW + Back.YELLOW + '||'+ Fore.BLACK + Back.WHITE + ' Author - TheOuterLinux (https://theouterlinux.gitlab.io)       ' + Fore.YELLOW + Back.YELLOW + '||' + Style.RESET_ALL)
	print(' ' + Fore.YELLOW + Back.YELLOW + '||'+ Fore.BLACK + Back.WHITE + ' License - GPL v3 or newer (https://www.gnu.org/licenses/)      ' + Fore.YELLOW + Back.YELLOW + '||' + Style.RESET_ALL)
	#print (' ' + Fore.YELLOW + Back.YELLOW + '+-------------------------------------------------------------------++' + Style.RESET_ALL)
	print(' ' + Fore.YELLOW + Back.YELLOW + "||"+ Fore.WHITE + Back.YELLOW + "           Use 'q' or 'quit' any time to end program.           " + Fore.YELLOW + Back.YELLOW + "||" + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '')
	linestart = raw_input(' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' On what line would you like to place the subroutine for the graphic? (' + Fore.WHITE + '1,100,..,1000,q' + Fore.YELLOW + '): ' + Style.RESET_ALL)
	linestart = linestart.lower()
	
	if linestart in ['q','quit','Q','Quit','QUIT','Exit','EXIT','exit']:
		rmtree(tmp, ignore_errors=True)
		print('')
		exit()
	else:
		clear()
		filenumber = startchr = 224
		vdunumber = 224
		filelist = sorted(os.listdir(tmp))
		if linestart in ['',' ']:
			print ('1000DEFPROC'+dosbase)
			print ('1001SPRX='+str(cursorw))
			print ('1002SPRY='+str(cursorh))
			linestart = 1003
		else:
			print (str(linestart)+'DEFPROC'+dosbase)
			linestart = int(linestart)+1
			print (str(linestart)+'SPRX='+str(cursorw))
			linestart = int(linestart)+1
			print (str(linestart)+'SPRY='+str(cursorh))
			linestart = int(linestart)+1
		print (str(linestart)+'VDU23,'+str(whiteblocknumber)+',&FF,&FF,&FF,&FF,&FF,&FF,&FF,&FF')
		previousBcheck = ''
		appendVDU = 'VDU31,SPRX,SPRY:VDU' #Using this removes the dependency of having to use this program just to move the graphic around on the screen.
		linebreak = width/8
		VDUSameCheck = ''
		for filename in filelist:
			if filename.endswith((".npy")):
				filenumber = filenumber + 1
				a = np.load(tmp + '/' + dosbase + '_' + str(filenumber) + '.npy')
				a = np.array2string(a, precision=1, separator=',',suppress_small=True, formatter={'int':lambda a:hex(int(a))}).replace('0x','&').upper()\
				.replace('&0,','0,')\
				.replace('&1,','1,')\
				.replace('&2,','2,')\
				.replace('&3,','3,')\
				.replace('&4,','4,')\
				.replace('&5,','5,')\
				.replace('&6,','6,')\
				.replace('&7,','7,')\
				.replace('&8,','8,')\
				.replace('&9,','9,')\
				.replace('&10,','16,')\
				.replace('&11,','17,')\
				.replace('&12,','18,')\
				.replace('&13,','19,')\
				.replace('&14,','20,')\
				.replace('&15,','21,')\
				.replace('&16,','22,')\
				.replace('&17,','23,')\
				.replace('&18,','24,')\
				.replace('&19,','25,')\
				.replace('&1A,','26,')\
				.replace('&1B,','27,')\
				.replace('&1C,','28,')\
				.replace('&1D,','29,')\
				.replace('&1E,','30,')\
				.replace('&1F,','31,')\
				.replace('&20,','32,')\
				.replace('&21,','33,')\
				.replace('&22,','34,')\
				.replace('&23,','35,')\
				.replace('&24,','36,')\
				.replace('&25,','37,')\
				.replace('&26,','38,')\
				.replace('&27,','39,')\
				.replace('&28,','40,')\
				.replace('&29,','41,')\
				.replace('&2A,','42,')\
				.replace('&2B,','43,')\
				.replace('&2C,','44,')\
				.replace('&2D,','45,')\
				.replace('&2E,','46,')\
				.replace('&2F,','47,')\
				.replace('&30,','48,')\
				.replace('&31,','49,')\
				.replace('&32,','50,')\
				.replace('&33,','51,')\
				.replace('&34,','52,')\
				.replace('&35,','53,')\
				.replace('&36,','54,')\
				.replace('&37,','55,')\
				.replace('&38,','56,')\
				.replace('&39,','57,')\
				.replace('&3A,','58,')\
				.replace('&3B,','59,')\
				.replace('&3C,','60,')\
				.replace('&3D,','61,')\
				.replace('&3E,','62,')\
				.replace('&3F,','63,')\
				.replace('&40,','64,')\
				.replace('&41,','65,')\
				.replace('&42,','66,')\
				.replace('&43,','67,')\
				.replace('&44,','68,')\
				.replace('&45,','69,')\
				.replace('&46,','70,')\
				.replace('&47,','71,')\
				.replace('&48,','72,')\
				.replace('&49,','73,')\
				.replace('&4A,','74,')\
				.replace('&4B,','75,')\
				.replace('&4C,','76,')\
				.replace('&4D,','77,')\
				.replace('&4E,','78,')\
				.replace('&4F,','79,')\
				.replace('&50,','80,')\
				.replace('&51,','81,')\
				.replace('&52,','82,')\
				.replace('&53,','83,')\
				.replace('&54,','84,')\
				.replace('&55,','85,')\
				.replace('&56,','86,')\
				.replace('&57,','87,')\
				.replace('&58,','88,')\
				.replace('&59,','89,')\
				.replace('&5A,','90,')\
				.replace('&5B,','91,')\
				.replace('&5C,','92,')\
				.replace('&5D,','93,')\
				.replace('&5E,','94,')\
				.replace('&5F,','95,')\
				.replace('&60,','96,')\
				.replace('&61,','97,')\
				.replace('&62,','98,')\
				.replace('&63,','99,')
				VDUCheck = str(a).replace(' ','').replace('[','').replace(']','')
				if  VDUCheck == '255,255,255,255,255,255,255,255':
					appendVDU = appendVDU + (str(hex(whiteblocknumber).replace('0x','&').upper())+';')
				elif VDUCheck == '0,0,0,0,0,0,0,&0':
					appendVDU = appendVDU + (str(blackblocknumber)+';')
				else:
					if VDUSameCheck != VDUCheck:
						if vdunumber == 255: 
							vdunumber = 127 #May or may not allow you to have an extra 32 set of ASCII characters to play around with.
							vdunumber = vdunumber+1
						else:
							vdunumber = vdunumber+1
					appendVDU = appendVDU + (str(hex(vdunumber).replace('0x','&').upper())+';')
					if VDUSameCheck != VDUCheck:
						linestart = int(linestart)+1
						print (str(linestart)+'VDU23,'+str(vdunumber)+','+str(a).replace(' ','').replace('[','').replace(']','')\
						.replace('&0,','0,')\
						.replace('&0','0')\
						.replace('&1,','1,')\
						.replace('&2,','2,')\
						.replace('&2,','2,')\
						.replace('&3,','3,')\
						.replace('&4,','4,')\
						.replace('&5,','5,')\
						.replace('&6,','6,')\
						.replace('&7,','7,')\
						.replace('&8,','8,')\
						.replace('&9,','9,'))
					VDUSameCheck = VDUCheck
				if appendVDU.count(';') % linebreak == 0: #Helps separate PRINT lines based on image width / 8 (because of 8x8 blocks)
					linestart = int(linestart)+1
					while (appendVDU[-3::] == '32;'):
						appendVDU = appendVDU[:-3]
					if appendVDU == 'SPRY=SPRY+1:VDU31,SPRX,SPRY:VDU':
						appendVDU = ''
						linestart = ''
					print (str(linestart)+appendVDU)
					appendVDU = 'SPRY=SPRY+1:VDU31,SPRX,SPRY:VDU'
					vdunumber = 224
if colornumber == 0:
	clear()
	print(' ' + Fore.YELLOW + Back.YELLOW + '+-----------------------------------------------------------------++' + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + '||'+ Fore.BLACK + Back.WHITE + ' img2vdu.py - Convert a modern image to BBC BASIC VDU 23 format ' + Fore.YELLOW + Back.YELLOW + '||' + Style.RESET_ALL)
	print(' ' + Fore.YELLOW + Back.YELLOW + '||'+ Fore.BLACK + Back.WHITE + ' Author - TheOuterLinux (https://theouterlinux.gitlab.io)       ' + Fore.YELLOW + Back.YELLOW + '||' + Style.RESET_ALL)
	print(' ' + Fore.YELLOW + Back.YELLOW + '||'+ Fore.BLACK + Back.WHITE + ' License - GPL v3 or newer (https://www.gnu.org/licenses/)      ' + Fore.YELLOW + Back.YELLOW + '||' + Style.RESET_ALL)
	#print (' ' + Fore.YELLOW + Back.YELLOW + '+-------------------------------------------------------------------++' + Style.RESET_ALL)
	print(' ' + Fore.YELLOW + Back.YELLOW + "||"+ Fore.WHITE + Back.YELLOW + "           Use 'q' or 'quit' any time to end program.           " + Fore.YELLOW + Back.YELLOW + "||" + Style.RESET_ALL)
	print (' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + '')
	linestart = raw_input(' ' + Fore.YELLOW + Back.YELLOW + ' ' + Back.BLACK + ' On what line would you like to place the graphic? (' + Fore.WHITE + '1,100,..,1000,q' + Fore.YELLOW + '): ' + Style.RESET_ALL)
	linestart = linestart.lower()
	if linestart in ['q','quit','Q','Quit','QUIT','Exit','EXIT','exit']:
		rmtree(tmp, ignore_errors=True)
		print('')
		exit()
	else:
		clear()
		if linestart in ['',' ']: linestart = 1000
		im = Image.open(previewfile)
		width, height = im.size
		cmd = ('python '+DIR+'/img2ascii.py --cols='+str(width)+' --file="'+filepath+'" --out="'+dirpath+'/'+base+'.ascii"')
		subprocess.call(cmd, shell=True)
		with open(dirpath+'/'+base+'.ascii') as f:
			content = f.readlines()
		with open(dirpath+'/'+base+'.ascii', 'w') as f:
			f.write(str(linestart)+' DEF PROC'+ dosbase + '\n')
			linestart = int(linestart)+1
			f.write(str(linestart)+' SPRX='+str(cursorw)+'\n')
			linestart = int(linestart)+1
			f.write(str(linestart)+' SPRY='+str(cursorh)+'\n')
			for line in content:
				linestart = int(linestart)+1
				f.write(str(linestart)+' SPRY=SPRY+1:PRINT TAB(SPRX,SPRY) "' + line.strip() + '"\n')
		os.remove(previewfile)

# Cleanup; deletes the temporary directory this script created at the beginning
rmtree(tmp, ignore_errors=True)

#Return colors back to normal on Windows
if os.name == 'Windows': deinit()
